@@ -1,7 +1,7 @@
 (********************************************************************)
 (*                                                                  *)
 (*  The Why3 Verification Platform   /   The Why3 Development Team  *)
-(*  Copyright 2010-2021 --  Inria - CNRS - Paris-Saclay University  *)
+(*  Copyright 2010-2022 --  Inria - CNRS - Paris-Saclay University  *)
 (*                                                                  *)
 (*  This software is distributed under the terms of the GNU Lesser  *)
 (*  General Public License version 2.1, with the special exception  *)
@@ -34,13 +34,13 @@ type proof_attempt_status =
 
 let print_status fmt st =
   match st with
-  | Undone            -> fprintf fmt "Undone"
-  | Scheduled         -> fprintf fmt "Scheduled"
-  | Running           -> fprintf fmt "Running"
+  | Undone            -> pp_print_string fmt "Undone"
+  | Scheduled         -> pp_print_string fmt "Scheduled"
+  | Running           -> pp_print_string fmt "Running"
   | Done r            -> fprintf fmt "Done(@[<h>%a@])"
                            (Call_provers.print_prover_result ~json:false) r
-  | Interrupted       -> fprintf fmt "Interrupted"
-  | Detached          -> fprintf fmt "Detached"
+  | Interrupted       -> pp_print_string fmt "Interrupted"
+  | Detached          -> pp_print_string fmt "Detached"
   | InternalFailure e ->
       fprintf fmt "InternalFailure(%a)" Exn_printer.exn_printer e
   | Uninstalled pr    ->
@@ -56,10 +56,10 @@ type transformation_status =
 
 let print_trans_status fmt st =
   match st with
-  | TSscheduled -> fprintf fmt "TScheduled"
-  | TSdone _tid -> fprintf fmt "TSdone" (* TODO print tid *)
-  | TSfailed _e -> fprintf fmt "TSfailed"
-  | TSfatal _e -> fprintf fmt "TSfatal"
+  | TSscheduled -> pp_print_string fmt "TScheduled"
+  | TSdone _tid -> pp_print_string fmt "TSdone" (* TODO print tid *)
+  | TSfailed _e -> pp_print_string fmt "TSfailed"
+  | TSfatal _e -> pp_print_string fmt "TSfatal"
 
 type strategy_status = STSgoto of proofNodeID * int | STShalt
                      | STSfatal of string * proofNodeID * exn
@@ -68,8 +68,8 @@ let print_strategy_status fmt st =
   match st with
   | STSgoto(id,n) ->
       fprintf fmt "goto step %d in proofNode %a" n print_proofNodeID id
-  | STShalt -> fprintf fmt "halt"
-  | STSfatal _ -> fprintf fmt "fatal"
+  | STShalt -> pp_print_string fmt "halt"
+  | STSfatal _ -> pp_print_string fmt "fatal"
 
 
 type controller =
@@ -304,9 +304,6 @@ let scheduled_proof_attempts : sched_pa_rec Queue.t = Queue.create ()
 (* type for prover tasks in progress *)
 type tasks_prog_rec =
   {
-    tp_session  : Session_itp.session;
-    tp_id       : proofNodeID;
-    tp_pr       : Whyconf.prover;
     tp_callback : (proof_attempt_status -> unit);
     tp_started  : bool;
     tp_call     : Call_provers.prover_call;
@@ -420,14 +417,14 @@ let build_prover_call spa =
                   (Pp.print_option Pp.string) spa.spa_pr_scr;
     let inplace = config_pr.Whyconf.in_place in
     let interactive = config_pr.Whyconf.interactive in
+    let main = Whyconf.get_main c.controller_config in
+    let libdir = Whyconf.libdir main in
+    let datadir = Whyconf.datadir main in
     try
       let call = Driver.prove_task ?old:spa.spa_pr_scr ~inplace ~command
-          ~limit ~interactive driver task in
+          ~limit ~interactive ~libdir ~datadir driver task in
       let pa =
-        { tp_session  = c.controller_session;
-          tp_id       = spa.spa_id;
-          tp_pr       = spa.spa_pr;
-          tp_callback = spa.spa_callback;
+        { tp_callback = spa.spa_callback;
           tp_started  = false;
           tp_call     = call;
           tp_ores     = spa.spa_ores } in
@@ -531,11 +528,12 @@ let idle_handler () =
   end;
   !idle_handler_running
 
-let interrupt () =
+let interrupt c =
+  let libdir = Whyconf.libdir (Whyconf.get_main c.controller_config) in
   (* Interrupt provers *)
   Hashtbl.iter
     (fun call e ->
-     Call_provers.interrupt_call call;
+     Call_provers.interrupt_call ~libdir call;
      e.tp_callback Interrupted)
     prover_tasks_in_progress;
   Hashtbl.clear prover_tasks_in_progress;
@@ -567,7 +565,7 @@ let run_idle_handler () =
       S.timeout ~ms:default_delay_ms timeout_handler;
     end
 
-let schedule_proof_attempt c id pr ?save_to ~limit ~callback ~notification =
+let schedule_proof_attempt ?proof_script_filename c id pr ~limit ~callback ~notification =
   let ses = c.controller_session in
   let callback panid s =
     begin
@@ -605,16 +603,16 @@ let schedule_proof_attempt c id pr ?save_to ~limit ~callback ~notification =
       let use_steps = Call_provers.(limit.limit_steps <> empty_limit.limit_steps) in
       let limit = adapt_limits ~interactive ~use_steps limit a in
       let script =
-        if save_to = None then
+        if proof_script_filename = None then
           Opt.map (fun s ->
               let s = Pp.sprintf "%a" Sysutil.print_file_path s in
               Debug.dprintf debug_sched "Script file = %s@." s;
               Filename.concat (get_dir ses) s) a.proof_script
-        else
-          save_to
+          else
+            proof_script_filename
       in
       limit, old_res, script
-    with Not_found | Session_itp.BadID -> limit,None,save_to
+    with Not_found | Session_itp.BadID -> limit,None,proof_script_filename
   in
   let panid = graft_proof_attempt ~limit ses id pr in
   let spa =
@@ -746,7 +744,10 @@ let schedule_edition c id pr ~callback ~notification =
   Debug.dprintf debug_sched "[Editing] goal %s with command '%s' on file %s@."
                 (Session_itp.get_proof_name session id).Ident.id_string
                 editor file;
-  let call = Call_provers.call_editor ~command:editor file in
+  let main = Whyconf.get_main config in
+  let libdir = Whyconf.libdir main in
+  let datadir = Whyconf.datadir main in
+  let call = Call_provers.call_editor ~command:editor ~libdir ~datadir file in
   callback panid Running;
   Queue.add (callback panid,call,old_res) prover_tasks_edited;
   run_idle_handler ()
@@ -805,10 +806,11 @@ let run_strategy_on_goal
       callback STShalt
     else
       match Array.get strat pc with
-      | Icall_prover(p,timelimit,memlimit) ->
+      | Icall_prover(p,timelimit,memlimit,steplimit) ->
          let main = Whyconf.get_main c.controller_config in
          let timelimit = Opt.get_def (Whyconf.timelimit main) timelimit in
          let memlimit = Opt.get_def (Whyconf.memlimit main) memlimit in
+         let steplimit = Opt.get_def 0 steplimit in
          let callback panid res =
            callback_pa panid res;
            match res with
@@ -827,10 +829,12 @@ let run_strategy_on_goal
                          (* should not happen *)
                          assert false
          in
-         let limit = { Call_provers.empty_limit with
+         let limit = {
                        Call_provers.limit_time = timelimit;
-                       limit_mem  = memlimit} in
-         schedule_proof_attempt c g p ?save_to:None ~limit ~callback ~notification
+                       limit_mem  = memlimit;
+                       limit_steps = steplimit;
+                     } in
+         schedule_proof_attempt c g p ~limit ~callback ~notification
       | Itransform(trname,pcsuccess) ->
          let callback ntr =
            callback_tr trname [] ntr;
@@ -852,7 +856,11 @@ let run_strategy_on_goal
                  S.idle ~prio:0 run_next)
                 (get_sub_tasks c.controller_session tid)
          in
-         schedule_transformation c g trname [] ~callback ~notification
+         begin match Session_itp.get_transformation c.controller_session g trname [] with
+         | tid -> callback (TSdone tid)
+         | exception Not_found ->
+             schedule_transformation c g trname [] ~callback ~notification
+         end
       | Igoto pc ->
          callback (STSgoto (g,pc));
          exec_strategy pc strat g
@@ -967,7 +975,7 @@ let rec copy_rec ~notification ~callback_pa ~callback_tr c from_any to_any =
  *)
     | APn from_pn, APn to_pn ->
       let from_pa_list = get_proof_attempts s from_pn in
-      List.iter (fun x -> schedule_pa_with_same_arguments ?save_to:None c x to_pn
+      List.iter (fun x -> schedule_pa_with_same_arguments c x to_pn
           ~callback:callback_pa ~notification) from_pa_list;
       let from_tr_list = get_transformations s from_pn in
       let callback x tr args st = callback_tr tr args st;
@@ -1064,7 +1072,7 @@ let replay_proof_attempt c pr limit (parid: proofNodeID) id ~callback ~notificat
      try
        if pr' <> pr then callback id (UpgradeProver pr');
        let _ = get_task c.controller_session parid in
-       schedule_proof_attempt ?save_to:None c parid pr' ~limit ~callback ~notification
+       schedule_proof_attempt c parid pr' ~limit ~callback ~notification
      with Not_found ->
        callback id Detached
 
@@ -1089,11 +1097,11 @@ let print_report fmt (r: report) =
   | CallFailed e ->
     Format.fprintf fmt "Callfailed %a" Exn_printer.exn_printer e
   | Replay_interrupted ->
-    Format.fprintf fmt "Interrupted"
+    Format.pp_print_string fmt "Interrupted"
   | Prover_not_installed ->
-    Format.fprintf fmt "Prover not installed"
+    Format.pp_print_string fmt "Prover not installed"
   | Edited_file_absent _ ->
-    Format.fprintf fmt "No edited file"
+    Format.pp_print_string fmt "No edited file"
   | No_former_result new_r ->
     Format.fprintf fmt "new_result = %a, no former result"
       (Call_provers.print_prover_result ~json:false) new_r
@@ -1217,11 +1225,12 @@ let create_rem_list =
     Buffer.add_string b (Pp.string_of pr id)
   in
   let module P = (val Pretty.create
+      ~do_forget_all:false
       rem.Eliminate_definition.rem_nt.Trans.printer
       rem.Eliminate_definition.rem_nt.Trans.aprinter
       rem.Eliminate_definition.rem_nt.Trans.printer
       rem.Eliminate_definition.rem_nt.Trans.printer
-      false) in
+      ) in
   let remove_ts ts = add P.print_ts ts in
   let remove_ls ls = add P.print_ls ls in
   let remove_pr pr = add P.print_pr pr in
@@ -1285,7 +1294,7 @@ let bisect_proof_attempt ~callback_tr ~callback_pa ~notification ~removed c pa_i
                        (Call_provers.print_prover_result ~json:false) res
                   end
                 in
-                schedule_proof_attempt ?save_to:None c pn prover ~limit ~callback ~notification
+                schedule_proof_attempt c pn prover ~limit ~callback ~notification
              | _ -> assert false
           end
         in
@@ -1367,7 +1376,7 @@ later on. We do has if proof fails. *)
             in
             Debug.dprintf
               debug "[Bisect] running the prover on subtask@.";
-            schedule_proof_attempt ?save_to:None c pn prover ~limit ~callback ~notification
+            schedule_proof_attempt c pn prover ~limit ~callback ~notification
          | _ -> assert false
       end
     in
