@@ -1,7 +1,7 @@
 (********************************************************************)
 (*                                                                  *)
 (*  The Why3 Verification Platform   /   The Why3 Development Team  *)
-(*  Copyright 2010-2021 --  Inria - CNRS - Paris-Saclay University  *)
+(*  Copyright 2010-2022 --  Inria - CNRS - Paris-Saclay University  *)
 (*                                                                  *)
 (*  This software is distributed under the terms of the GNU Lesser  *)
 (*  General Public License version 2.1, with the special exception  *)
@@ -29,17 +29,17 @@ let opt_metas = ref []
 (* Option for printing counterexamples with JSON formatting *)
 let opt_json : [< `All | `Values ] option ref = ref None
 let opt_check_ce_model = ref false
-let opt_print_original_model = ref false
-let opt_print_derived_model = ref false
 let opt_rac_prover = ref None
-let opt_rac_try_negate = ref false
+let opt_rac_timelimit = ref None
+let opt_rac_steplimit = ref None
 let opt_ce_check_verbosity = ref None
+let opt_sub_goals = ref []
 
-let () = (* Instead of additional command line parameters *)
-  if Opt.get_def "" (Sys.getenv_opt "WHY3PRINTORIGINALMODEL") = "yes" then
-    opt_print_original_model := true;
-  if Opt.get_def "" (Sys.getenv_opt "WHY3PRINTDERIVEDMODEL") = "yes" then
-    opt_print_derived_model := true
+let debug_print_original_model = Debug.register_info_flag "print-original-model"
+    ~desc:"Print original counterexample model when --check-ce"
+
+let debug_print_derived_model = Debug.register_info_flag "print-derived-model"
+    ~desc:"Print derived counterexample model when --check-ce"
 
 let add_opt_file x =
   let tlist = Queue.create () in
@@ -54,9 +54,9 @@ let add_opt_theory x =
   in
   match !opt_input, p with
   | None, [] ->
-      eprintf "Option '-T'/'--theory' with a non-qualified \
-        argument requires an input file.@.";
-      exit 1
+      let msg = "Option '-T'/'--theory' with a non-qualified \
+                 argument requires an input file.@." in
+      raise (Getopt.GetoptFailure msg)
   | Some tlist, [] ->
       let glist = Queue.create () in
       let elist = Queue.create () in
@@ -73,9 +73,10 @@ let add_opt_theory x =
 
 let add_opt_goal x =
   let glist = match !opt_theory, !opt_input with
-    | None, None -> eprintf
-        "Option '-G'/'--goal' requires an input file or a library theory.@.";
-        exit 1
+    | None, None ->
+        let msg = "Option '-G'/'--goal' requires an input file or a library \
+                   theory.@." in
+        raise (Getopt.GetoptFailure msg)
     | None, Some _ ->
         add_opt_theory "Top";
         Opt.get !opt_theory
@@ -83,7 +84,11 @@ let add_opt_goal x =
   let l = Strings.split '.' x in
   Queue.push (x, l) glist
 
-let add_opt_trans x = opt_trans := x::!opt_trans
+let add_opt_trans x =
+  match String.split_on_char ' ' x with
+  | [] -> assert false
+  | name :: args ->
+      opt_trans := (name, args) :: !opt_trans
 
 let add_opt_meta meta =
   let meta_name, meta_arg =
@@ -96,6 +101,30 @@ let add_opt_meta meta =
   in
   opt_metas := (meta_name,meta_arg)::!opt_metas
 
+let subgoal_re = Re.Str.regexp "^\\([^:@]+\\)?\\(:[^@]+\\)?\\(@.+\\)?$"
+
+let add_sub_goal s =
+  let failure str =
+    let msg = str ^ " for option --sub-goal" in
+    raise (Getopt.GetoptFailure msg) in
+  if Re.Str.string_match subgoal_re s 0 then (
+    let f =
+      try Re.Str.matched_group 1 s
+      with Not_found ->
+      try Opt.get (fst (Queue.peek opt_queue))
+      with _ -> failure "Missing file" in
+    let l =
+      try
+        let s = Strings.remove_prefix ":" (Re.Str.matched_group 2 s) in
+        Some (int_of_string s)
+      with Not_found -> None | Failure _ -> failure "Invalid line number" in
+    let e =
+      try Some (Strings.remove_prefix "@" (Re.Str.matched_group 3 s))
+      with Not_found -> None in
+    opt_sub_goals := (f,l,e) :: !opt_sub_goals )
+  else
+    failure "Invalid argument"
+
 let opt_driver = ref []
 let opt_parser = ref None
 let opt_prover = ref None
@@ -115,6 +144,12 @@ let option_list =
     "<theory> select <theory> in the input file or in the library";
     Key ('G', "goal"), Hnd1 (AString, add_opt_goal),
     "<goal> select <goal> in the last selected theory";
+    Key ('a', "apply-transform"), Hnd1 (AString, add_opt_trans),
+    "<transf> apply a transformation to every task";
+    Key ('g', "sub-goal"), Hnd1 (AString, add_sub_goal),
+    "[<file>][:<line>][@<expl>] select sub-goals at the given\n\
+     position and with the given explanation after applying\n\
+     the transformations (<file> defaults to the input file)";
     Key ('P', "prover"), Hnd1 (AString, fun s -> opt_prover := Some s),
     "<prover> prove or print (with -o) the selected goals";
     Key ('F', "format"), Hnd1 (AString, fun s -> opt_parser := Some s),
@@ -125,8 +160,6 @@ let option_list =
     "<steps> set the prover's step limit (default: no limit)";
     Key ('m', "memlimit"), Hnd1 (AInt, fun i -> opt_memlimit := Some i),
     "<MiB> set the prover's memory limit (default: no limit)";
-    Key ('a', "apply-transform"), Hnd1 (AString, add_opt_trans),
-    "<transf> apply a transformation to every task";
     Key ('M', "meta"), Hnd1 (AString, add_opt_meta),
     "<meta>[=<string>] add a meta to every task";
     Key ('D', "driver"), Hnd1 (AString, fun s -> opt_driver := s::!opt_driver),
@@ -147,11 +180,12 @@ let option_list =
      (RAC)";
     KLong "rac-prover", Hnd1 (AString, fun s -> opt_rac_prover := Some s),
     "<prover> use <prover> to check assertions in RAC when term\n\
-     reduction is insufficient, with optional, comma-\n\
-     separated time and memory limit (e.g. 'cvc4,2,1000')";
-    KLong "rac-try-negate", Hnd0 (fun () -> opt_rac_try_negate := true),
-    " try checking the negated term using the RAC prover when\n\
-     the prover is defined and didn't give a result";
+     reduction is insufficient, with optional, space-\n\
+     separated time and memory limit (e.g. 'cvc4 2 1000')";
+    KLong "rac-timelimit", Hnd1 (AInt, fun i -> opt_rac_timelimit := Some i),
+    "<seconds> Time limit in seconds for RAC (with --check-ce)";
+    KLong "rac-steplimit", Hnd1 (AInt, fun i -> opt_rac_steplimit := Some i),
+    "<seconds> Step limit for RAC (with --check-ce)";
     Key ('v',"verbosity"), Hnd1(AInt, fun i -> opt_ce_check_verbosity := Some i),
     "<lvl> verbosity level for interpretation log of counterexam-\n\
      ple solver model";
@@ -218,9 +252,11 @@ let () = try
   end;
   let add_meta task (meta,s) =
     let meta = lookup_meta meta in
-    let args = match s with
-      | Some s -> [MAstr s]
-      | None -> []
+    let args = match meta.meta_type, s with
+      | [MTstring], Some s -> [MAstr s]
+      | [MTint], Some s -> [MAint (int_of_string s)]
+      | [], None -> []
+      | _ -> failwith "meta argument not implemented"
     in
     Task.add_meta task meta args
   in
@@ -245,6 +281,20 @@ let memlimit = match !opt_memlimit with
 let print_th_namespace fmt th =
   Pretty.print_namespace fmt th.th_name.Ident.id_string th
 
+let really_do_task (task: task) =
+  let t = task_goal_fmla task in
+  let aux (f,l,e) =
+    match t.Term.t_loc with
+    | None -> false
+    | Some loc ->
+        let goal_f, goal_l, _, _ = Loc.get loc in
+        goal_f = f &&
+        (match l with None -> true | Some l -> l = goal_l) &&
+        (match e with None -> true | Some e ->
+         let expls = String.concat " " (Termcode.get_expls_fmla t) in
+         String.(equal (lowercase_ascii e) (lowercase_ascii expls))) in
+  !opt_sub_goals = [] || List.exists aux !opt_sub_goals
+
 let fname_printer = ref (Ident.create_ident_printer [])
 
 let output_task drv fname _tname th task dir =
@@ -265,81 +315,100 @@ let print_result ?json fmt (fname, loc, goal_name, expls, res, ce) =
   match json with
   | Some `All ->
     let open Json_base in
-    let print_loc fmt (loc, fname) =
+    let loc =
       match loc with
-      | None -> fprintf fmt "{%a}" (print_json_field "filename" print_json) (String fname)
-      | Some loc -> Pretty.print_json_loc fmt loc in
-    let print_term fmt (loc, fname, goal_name, expls) =
-      fprintf fmt "@[@[<hv1>{%a;@ %a;@ %a@]}@]"
-        (print_json_field "loc" print_loc) (loc, fname)
-        (print_json_field "goal_name" print_json) (String goal_name)
-        (print_json_field "explanations" print_json) (List (List.map (fun s -> String s) expls)) in
-    fprintf fmt "@[@[<hv1>{%a;@ %a@]}@]"
-      (print_json_field "term" print_term) (loc, fname, goal_name, expls)
-      (print_json_field "prover-result" (Call_provers.print_prover_result ~json:true)) res
+      | None -> Record ["filename", String fname]
+      | Some loc ->
+          let f, l, b, e = Loc.get loc in
+          Record [
+              "filename", String f;
+              "line", Int l;
+              "start-char", Int b;
+              "end-char", Int e
+            ] in
+    let term =
+      Record [
+          "loc", loc;
+          "goal_name", String goal_name;
+          "explanations", List (List.map (fun s -> String s) expls)
+        ] in
+    print_json fmt
+      (Record [
+           "term", term;
+           "prover-result", Call_provers.json_prover_result res
+         ])
   | None | Some `Values as json ->
     ( match loc with
       | None -> fprintf fmt "File %s:@\n" fname
       | Some loc -> Loc.report_position fmt loc );
     ( if expls = [] then
-        fprintf fmt "@[<hov>Verification@ condition@ @{<bold>%s@}.@]" goal_name
+        fprintf fmt "@[<hov>Goal@ @{<bold>%s@}.@]" goal_name
       else
         let expls = String.capitalize_ascii (String.concat ", " expls) in
         fprintf fmt
-          "@[<hov>Goal@ @{<bold>%s@}@ from@ verification@ condition@ @{<bold>%s@}.@]"
+          "@[<hov>Sub-goal@ @{<bold>%s@}@ of@ goal@ @{<bold>%s@}.@]"
           expls goal_name );
     fprintf fmt "@\n@[<hov2>Prover result is: %a.@]"
       (Call_provers.print_prover_result ~json:false) res;
-    (match ce with
-     | Some ce ->
-        Counterexample.print_counterexample ?verb_lvl:!opt_ce_check_verbosity
-          ~check_ce:!opt_check_ce_model ?json fmt ce
-     | None -> ());
+    Opt.iter
+      (Check_ce.print_model_classification ?json
+         ?verb_lvl:!opt_ce_check_verbosity ~check_ce:!opt_check_ce_model fmt)
+      ce;
     fprintf fmt "@\n"
 
 let unproved = ref false
 
+let select_ce env th models =
+  if models <> [] then
+    match Pmodule.restore_module th with
+    | pm ->
+        let rac = Pinterp.mk_rac ~ignore_incomplete:false
+            (Rac.Why.mk_check_term_lit config env ?why_prover:!opt_rac_prover ()) in
+        let timelimit = Opt.map float_of_int !opt_rac_timelimit in
+        Check_ce.select_model
+          ?timelimit ?steplimit:!opt_rac_steplimit ?verb_lvl:!opt_ce_check_verbosity
+          ~check_ce:!opt_check_ce_model rac env pm models
+    | exception Not_found -> None
+  else None
+
+let print_other_models (m, (c, log)) =
+  let print_model fmt m =
+    let print_attrs = Debug.(test_flag (lookup_flag "print_model_attrs"))  in
+    if !opt_json = None then Model_parser.print_model_human fmt m ~print_attrs
+    else Model_parser.print_model (* json values *) fmt m ~print_attrs in
+  ( match c with
+    | Check_ce.(NC | SW | NC_SW | BAD_CE _) ->
+        if Debug.test_flag debug_print_original_model then
+          printf "@[<v>Original model:@\n%a@]@\n@." print_model m;
+    | _ -> () );
+  ( match c with
+    | Check_ce.(NC | SW | NC_SW) ->
+        if Debug.test_flag debug_print_derived_model then
+          printf "@[<v>Derived model:@\n%a@]@\n@." print_model
+            (Check_ce.model_of_exec_log ~original_model:m log)
+    | _ -> () )
+
 let do_task env drv fname tname (th : Theory.theory) (task : Task.task) =
+  if really_do_task task then
   let open Call_provers in
   let limit =
     { limit_time = timelimit;
       limit_mem = memlimit;
       limit_steps = stepslimit } in
+  let libdir = Config.libdir in
+  let datadir = Config.datadir in
   match !opt_output, !opt_command with
     | None, Some command ->
-        let call = Driver.prove_task ~command ~limit drv task in
+        let call =
+          Driver.prove_task ~command ~libdir ~datadir ~limit drv task
+        in
         let res = wait_on_call call in
-        let ce =
-          if res.pr_models <> [] then
-            match Pmodule.restore_module th with
-            | pm ->
-               let reduce_config =
-                 Pinterp.rac_reduce_config_lit config env
-                   ~trans:"compute_in_goal" ?prover:!opt_rac_prover
-                   ~try_negate:!opt_rac_try_negate () in
-               Counterexample.select_model ~reduce_config
-                 ~check:!opt_check_ce_model ?verb_lvl:!opt_ce_check_verbosity
-                 env pm res.pr_models
-            | exception Not_found -> None
-          else None in
+        let ce = select_ce env th res.pr_models in
         let t = task_goal_fmla task in
         let expls = Termcode.get_expls_fmla t in
         let goal_name = (task_goal task).Decl.pr_name.Ident.id_string in
         printf "%a@." (print_result ?json:!opt_json)
           (fname, t.Term.t_loc, goal_name, expls, res, ce);
-        let print_model fmt m =
-          let print_attrs = Debug.(test_flag (lookup_flag "print_model_attrs"))  in
-          if !opt_json = None then Model_parser.print_model_human fmt m ~print_attrs
-          else Model_parser.print_model (* json values *) fmt m ~print_attrs in
-        let print_other_models (m, ce_summary) =
-          match ce_summary with
-            | Counterexample.(NCCE log | SWCE log | NCCE_SWCE log) ->
-                if !opt_print_original_model then
-                  printf "@[<v>Original model:@\n%a@]@\n@." print_model m;
-                if !opt_print_derived_model then
-                  printf "@[<v>Derived model:@\n%a@]@\n@." print_model
-                    (Counterexample.model_of_exec_log ~original_model:m log)
-            | _ -> () in
         Opt.iter print_other_models ce;
         if res.pr_answer <> Valid then unproved := true
     | None, None ->
@@ -347,16 +416,18 @@ let do_task env drv fname tname (th : Theory.theory) (task : Task.task) =
     | Some dir, _ -> output_task drv fname tname th task dir
 
 let do_tasks env drv fname tname th task =
-  let lookup acc t =
-    (try Trans.singleton (Trans.lookup_transform t env) with
-       Trans.UnknownTrans _ -> Trans.lookup_transform_l t env) :: acc
-  in
-  let trans = List.fold_left lookup [] !opt_trans in
-  let apply tasks tr =
-    List.rev (List.fold_left (fun acc task ->
-      List.rev_append (Trans.apply tr task) acc) [] tasks)
-  in
-  let tasks = List.fold_left apply [task] trans in
+  let table = Args_wrapper.build_naming_tables task in
+  let rec apply tasks = function
+    | [] -> tasks
+    | (name, args) :: trans ->
+        let apply_trans =
+          if args = [] then
+            Trans.apply_transform name env
+          else
+            let ffmt = Env.get_format ?format:!opt_parser fname in
+            Trans.apply_transform_args name env args table ffmt in
+        apply (List.concat (List.map apply_trans tasks)) trans in
+  let tasks = apply [task] !opt_trans in
   List.iter (do_task env drv fname tname th) tasks
 
 let do_theory env drv fname tname th glist elist =
